{
  SHORTCTS.PAS   v1.01
  Copyright (C) 1999  Alexander Burlakov
            mailto:hunterml@fnmail.com
            http://www.chat.ru/~hunterml

            TShortCut - invisible Delphi 4 component for
                        easy Windows Shortcuts creation in your
                        application.


  Last Changed 15 Jul 1999
     - Added OnChanged and OnSynchronized events;
     - Added Icon read only property;
     - Fixed bug with LNK loading from file.

  If anyone using my component with Windows NT please, inform me how it
  works. I really need this information since I still unable to test this
  unit with NT.

  Please, email me your comments or/and suggestions.
  Thanx.
}

unit Shortcts;

interface

uses
  Windows, SysUtils, Classes, Forms, Graphics;

type
  TPlace = (plExactFolder, plStartMenu, plProgramsMenu, plDesktop, plStartUp, plRecent);
  TPlaces = set of TPlace;
  TFolderBase = (fbNone, fbWindows, fbStartMenu, fbProgramsMenu, fbDesktop, fbApplicationPath);
  TShowCmd = (cmdShowNormal, cmdShowMinimized, cmdShowMaximized);


  TShortCut = class(TComponent)
  private
    FActive : boolean;
// IShellLink parametrs
    FDescription : string;
    FIconPath : string;
    FIconIndex : cardinal;
    FPath : string;
    FWorkDir : string;
    FArguments : string;
    FShowCmd : TShowCmd;
// end
    FIcon : TIcon;
    FPlace : TPlaces;
    FFolder : string;
    FFolderBase : TFolderBase;
    FOnChanged : TNotifyEvent;
    FOnActivate : TNotifyEvent;
    FOnDeactivate : TNotifyEvent;
    FOnSynchronized : TNotifyEvent;
    function GetName_ExactFolder : string;
    function GetName_SpecialFolder(AFolder : integer) : string;
    function GetIconHandle : TIcon;
    procedure SetActive(Value : boolean);
    procedure SetPlace(Value : TPlaces);
    procedure SetDescription(Value : string);
    procedure SetIconPath(Value : string);
    procedure SetIconIndex(Value : cardinal);
    procedure SetPath(Value : string);
    procedure SetWorkingDirectory(Value : string);
    procedure SetArguments(Value : string);
    procedure SetShowCmd(Value : TShowCmd);
  protected
    procedure DoChanged;
    procedure DoActivateDeactivate;
    procedure SyncronizeShortcut;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure SaveToFile(FileName : string);
    procedure LoadFromFile(FileName : string);
    property Icon : TIcon read GetIconHandle;
  published
    property Active : boolean read FActive write SetActive;
    property Description : string read FDescription write SetDescription;
    property IconPath : string read FIconPath write SetIconPath;
    property IconIndex : cardinal read FIconIndex write SetIconIndex;
    property Path : string read FPath write SetPath;
    property WorkingDirectory : string read FWorkDir write SetWorkingDirectory;
    property Arguments : string read FArguments write SetArguments;
    property ShowCmd : TShowCmd read FShowCmd write SetShowCmd;
    property Place : TPlaces read FPlace write SetPlace;
    property ExactFolder : string read FFolder write FFolder;
    property ExactFolderBase : TFolderBase read FFolderBase write FFolderBase;
    property OnChanged : TNotifyEvent read FOnChanged write FOnChanged;
    property OnActivate : TNotifyevent read FOnActivate write FOnActivate;
    property OnDeactivate : TNotifyevent read FOnDeactivate write FOnDeactivate;
    property OnSynchronized : TNotifyEvent read FOnSynchronized write FOnSynchronized;
  end;


procedure Register;

{$IFNDEF VER120}    { This function not present in Delphi 3 }
function SHGetSpecialFolderPath(hwndOwner: HWND; lpszPath: PChar;
  nFolder: Integer; fCreate: BOOL): BOOL; stdcall;
{$ENDIF}


implementation
uses ShellAPI, ShlObj, Ole2, FileCtrl;
const
  extLNK               = '.lnk';

{$IFNDEF VER120}
function SHGetSpecialFolderPath;    external 'shell32.dll' name 'SHGetSpecialFolderPathA';
{$ENDIF}

function SlashSep(const Path, S: String): String;
begin
  Result := '';
  if Path = '' then
  begin
    Result := S;
    exit;
  end;
  if AnsiLastChar(Path)^ <> '\' then
    Result := Path + '\' + S
  else
    Result := Path + S;
end;

procedure RemoveEmptyDirectories(ADir : string);
begin
  if ADir[Length(ADir)] = '\' then Delete(ADir, Length(ADir), 1);
  while DirectoryExists(ADir) and RemoveDir(ADir) do
  begin
    while ADir[Length(ADir)] <> '\' do Delete(ADir, Length(ADir), 1);
    Delete(ADir, Length(ADir), 1);
  end;
end;

{ TShortCut }

constructor TShortCut.Create;
begin
  inherited;
  FActive := false;
  FDescription := '';
  FIconPath := '';
  FIconIndex := 1;
  FPath := '';
  FWorkDir := '';
  FArguments := '';
  FShowCmd := cmdShowNormal;
  FPlace := [plProgramsMenu];
  FFolder := '';
  FFolderBase := fbProgramsMenu;
  FOnChanged := nil;
end;

destructor TShortCut.Destroy;
begin
  if Assigned(FIcon) then FIcon.Free;
  inherited;
end;

procedure TShortCut.Assign(Source: TPersistent);
begin
  if Source is TShortCut then
  begin
    Description := TShortCut(Source).Description;
    IconPath := TShortCut(Source).IconPath;
    IconIndex := TShortCut(Source).IconIndex;
    Path := TShortCut(Source).Path;
    WorkingDirectory := TShortCut(Source).WorkingDirectory;
    Arguments := TShortCut(Source).Arguments;
    ShowCmd  := TShortCut(Source).ShowCmd;
    Place := TShortCut(Source).Place;
    ExactFolder := TShortCut(Source).ExactFolder;
    ExactFolderBase := TShortCut(Source).ExactFolderBase;
    Active := TShortCut(Source).Active;
  end
  else inherited Assign(Source);
end;

procedure TShortCut.DoChanged;
begin
  if Assigned(FOnChanged) then FOnChanged(Self);
end;

procedure TShortCut.DoActivateDeactivate;
begin
  if FActive then
  begin
    if Assigned(FOnActivate) then FOnActivate(Self);
  end else
  begin
    if Assigned(FOnDeactivate) then FOnDeactivate(Self);
  end;
end;

procedure TShortCut.Loaded;
begin
  inherited;
  DoChanged;
end;

procedure TShortCut.SyncronizeShortcut;
begin
  if FActive then
  begin
    if plExactFolder in FPlace then SaveToFile(GetName_ExactFolder) else
       begin
         DeleteFile(GetName_ExactFolder);
         RemoveEmptyDirectories(ExtractFilePath(GetName_ExactFolder));
       end;
    if plStartMenu in FPlace then SaveToFile(GetName_SpecialFolder(CSIDL_STARTMENU))
       else DeleteFile(GetName_SpecialFolder(CSIDL_STARTMENU));
    if plProgramsMenu in FPlace then SaveToFile(GetName_SpecialFolder(CSIDL_PROGRAMS))
       else DeleteFile(GetName_SpecialFolder(CSIDL_PROGRAMS));
    if plDesktop in FPlace then SaveToFile(GetName_SpecialFolder(CSIDL_DESKTOPDIRECTORY))
       else DeleteFile(GetName_SpecialFolder(CSIDL_DESKTOPDIRECTORY));
    if plStartUp in FPlace then SaveToFile(GetName_SpecialFolder(CSIDL_STARTUP))
       else DeleteFile(GetName_SpecialFolder(CSIDL_STARTUP));
    if plRecent in FPlace then SaveToFile(GetName_SpecialFolder(CSIDL_RECENT))
       else DeleteFile(GetName_SpecialFolder(CSIDL_RECENT));
  end;
  if Assigned(FOnSynchronized) then FOnSynchronized(self);
end;

procedure TShortCut.SaveToFile;
var
  SL : IShellLink;
  PF : IPersistFile;
  Str : array[0..1024] of char;
begin
  CoInitialize(nil);  // OLE initialization
  if Succeeded(COCreateInstance(OLE2.TGUID(CLSID_ShellLink), nil,
               CLSCTX_INPROC_SERVER, OLE2.TGUID(IID_IShellLinkA), SL)) then
  begin
    if Succeeded(SL.QueryInterface(System.TGUID(IID_IPersistFile), PF)) then
    begin
      SL.SetDescription(StrPCopy(Str, FDescription));
      if FPath = '' then FPath := Application.ExeName;
      SL.SetPath(StrPCopy(Str, FPath));
      if FIconPath = '' then  FIconPath := FPath;
      Sl.SetIconLocation(StrPCopy(Str, FIconPath), FIconIndex);
      SL.SetWorkingDirectory(StrPCopy(Str, FWorkDir));
      SL.SetArguments(StrPCopy(Str, FArguments));
      case FShowCmd of
        cmdShowNormal    : SL.SetShowCmd(SW_SHOWNORMAL);
        cmdShowMinimized : SL.SetShowCmd(SW_SHOWMINNOACTIVE);
        cmdShowMaximized : SL.SetShowCmd(SW_SHOWMAXIMIZED);
      end; { CASE }
      StringToWideChar(FileName, @Str[0], 512);
      if not Succeeded(PF.Save(@Str[0], true)) then raise
         Exception.Create(Format('IPersistFile.Save - Link creation error!'#13+
                          'File Name "%s"', [FileName]));
    end else raise Exception.Create('IPersistFile initialization error!');
  end else raise Exception.Create('IShellLink initialization error!');
end;

procedure TShortCut.LoadFromFile;
const
  MAX_STR    = 1024;
var
  SL : IShellLink;
  PF : IPersistFile;
  Str : array[0..MAX_STR] of char;
  Pfd : TWin32FindData;
  I : integer;
begin
  CoInitialize(nil);  // OLE initialization
  if Succeeded(COCreateInstance(OLE2.TGUID(CLSID_ShellLink), nil,
               CLSCTX_INPROC_SERVER, OLE2.TGUID(IID_IShellLinkA), SL)) then
  begin
    if Succeeded(SL.QueryInterface(System.TGUID(IID_IPersistFile), PF)) then
    begin
      StringToWideChar(FileName, @Str[0], 512);
      if not Succeeded(PF.Load(@Str[0], fmOpenRead and fmShareDenyNone)) then
      begin
        raise Exception.Create(Format('IPersistFile.Load - Link loading error!'#13+
                                      'File name "%s"', [FileName]));
        Exit;
      end;
      SL.GetDescription(@Str[0], MAX_STR);
           // Here is a bug! :( The SL.GetDescription does not work.
           // Could anyone tell me why ???
      FDescription := StrPas(@Str[0]);
      if FDescription = '' then
      begin
        FDescription := ExtractFileName(FileName);     // Remove LNK file extension
        if Copy(FDescription, Length(FDescription)-3, 1) = '.' then
           Delete(FDescription, Length(FDescription)-3, 4);
      end;
      SL.GetPath(@Str[0], MAX_STR, Pfd, 0);
      FPath := StrPas(@Str[0]);
      Sl.GetIconLocation(@Str[0], MAX_STR, integer(FIconIndex));
      FIconPath := StrPas(@Str[0]);
      SL.GetWorkingDirectory(@Str[0], MAX_STR);
      FWorkDir :=  StrPas(@Str[0]);
      SL.GetArguments(@Str[0], MAX_STR);
      FArguments :=  StrPas(@Str[0]);
      SL.GetShowCmd(I);
      case I of
        SW_SHOWNORMAL : FShowCmd := cmdShowNormal;
        SW_SHOWMINNOACTIVE : FShowCmd := cmdShowMinimized;
        SW_SHOWMAXIMIZED : FShowCmd := cmdShowMaximized;
      end; { CASE }
      FActive := false;      // turn off the Active for safety
      DoActivateDeactivate;  // call OnDeactivate event
      FFolder := ExtractFilePath(FileName);
      FFolderBase := fbNone;
      FPlace := [plExactFolder];
    end else raise Exception.Create('IPersistFile initialization error!');
  end else raise Exception.Create('IShellLink initialization error!');
  DoChanged;
end;

function TShortCut.GetName_ExactFolder : string;
var
  Dir : string;

  function GetBaseExactFolder : string;
  var
    Str : array[0..1024] of char;
  begin
    case FFolderBase of
      fbWindows :
         GetWindowsDirectory(@Str[0], 1024);
      fbStartMenu :
         SHGetSpecialFolderPath(Application.Handle, @Str[0], CSIDL_STARTMENU, true);
      fbProgramsMenu :
         SHGetSpecialFolderPath(Application.Handle, @Str[0], CSIDL_PROGRAMS, true);
      fbDesktop :
         SHGetSpecialFolderPath(Application.Handle, @Str[0], CSIDL_DESKTOPDIRECTORY, true);
      fbApplicationPath :
         begin
           Result := ExtractFilePath(Application.ExeName);
           exit;
         end;
      fbNone :
         begin
           Result := '';
           exit;
         end;
    end; { CASE }
    Result := StrPas(@Str[0]);
  end;

begin
  Dir := SlashSep(GetBaseExactFolder, FFolder);
  if not DirectoryExists(Dir) then ForceDirectories(Dir);
  Result := SlashSep(GetBaseExactFolder, SlashSep(FFolder, FDescription))+extLNK
end;

function TShortCut.GetName_SpecialFolder(AFolder : integer) : string;
var
  Str : array[0..1024] of char;
begin
  SHGetSpecialFolderPath(Application.Handle, @Str[0], AFolder, true);
  Result := SlashSep(StrPas(@Str[0]), FDescription)+extLNK;
end;

function TShortCut.GetIconHandle;
var
  Str : array[0..1024] of char;
  Idx : word;
  Hdl : HICON;
begin
  if not Assigned(FIcon) then FIcon := TIcon.Create;
  Hdl := 0;
  if FileExists(FIconPath) then
  begin
    Hdl := ExtractIcon(HInstance, StrPCopy(Str, FIconPath), FIconIndex);
  end;
  if Hdl = 0 then
     Hdl := ExtractAssociatedIcon(HInstance, StrPCopy(Str, FPath), Idx);
  FIcon.Handle := Hdl;
  Result := FIcon;
end;

procedure TShortCut.SetActive;
begin
  if Value <> FActive then
  begin
    FActive := Value;
    DoActivateDeactivate;
    SyncronizeShortcut;
    DoChanged;
  end;
end;

procedure TShortCut.SetPlace;
begin
  if Value <> FPlace then
  begin
    FPlace := Value;
    SyncronizeShortcut;
    DoChanged;
  end;
end;

procedure TShortCut.SetDescription(Value : string);
begin
  if Value <> FDescription then
    begin    FDescription := Value;    DoChanged;  end;
end;

procedure TShortCut.SetIconPath(Value : string);
begin
  if Value <> FIconPath then
    begin    FIconPath := Value;    DoChanged;  end;
end;

procedure TShortCut.SetIconIndex(Value : cardinal);
begin
  if Value <> FIconIndex then
    begin    FIconIndex := Value;    DoChanged;  end;
end;

procedure TShortCut.SetPath(Value : string);
begin
  if Value <> FPath then
    begin    FPath := Value;    DoChanged;  end;
end;

procedure TShortCut.SetWorkingDirectory(Value : string);
begin
  if Value <> FWorkDir then
    begin    FWorkDir := Value;    DoChanged;  end;
end;

procedure TShortCut.SetArguments(Value : string);
begin
  if Value <> FArguments then
    begin    FArguments := Value;    DoChanged;  end;
end;

procedure TShortCut.SetShowCmd(Value : TShowCmd);
begin
  if Value <> FShowCmd then
    begin    FShowCmd := Value;    DoChanged;  end;
end;



procedure Register;
begin
  RegisterComponents('Technisoft', [TShortCut]);
end;

end.
